require "msgpack"

class RRSE::Database
  def initialize(dir, separator)
    @table = File.open(dir.join("table")){|f| MessagePack.load(f)}
    @index_table = File.open(dir.join("index_table")){|f| MessagePack.load(f)}
    @separator = separator
  end

  def query_instance_method(query)
    @index_table.fetch(query, []).find_all{|spec|
      instance_method_name?(spec)
    }.map{|spec|
      query_full_name(spec)
    }.join(@separator)
  end

  def instance_method_name?(spec)
    return false unless /([.:#])[^.:#]+\Z/.match(spec)
    return $1 == "#"
  end

  def query_full_name(name)
    @table.fetch(name, []).map{|param| "#{name}#{param}"}.join(@separator)
  end

  def query_toplevel_method(query)
    @index_table.fetch(query, []).find_all{|spec|
      toplevel_name?(spec)
    }.map{|spec|
      query_full_name(spec)
    }.join(@separator)
  end

  def toplevel_name?(spec)
    /\A(Object\#|Kernel\.|Module#)(?!(initialize|class))/ =~ spec
  end
end
