require "rrse"
require "optparse"
require "pathname"

require_relative "setup-command"
require_relative "update-db-command"
require_relative "install-default-db-command"
require_relative "standalone-query-command"
require_relative "pipe-query-command"

class RRSE::Main
  DEFAULT_DIR = Pathname(ENV["HOME"]) + ".rrse"

  SUBCOMMANDS = {
    "setup-config" => RRSE::Command::SetUpConfig,
    "update-db" => RRSE::Command::UpdateDB,
    "install-default-db" =>  RRSE::Command::InstallDefaultDB,
    "standalone-query" => RRSE::Command::StandaloneQuery,
    "pipe-query" => RRSE::Command::PipeQuery,
  }

  def self.show_help
    print(self.new.options.help)
  end
  
  def initialize
    @rrse_dir = DEFAULT_DIR
  end

  def options
    opts = OptionParser.new
    opts.program_name = "RRSE"
    opts.version = RRSE::VERSION
    opts.banner = <<EOS
Usage: rrse [global-options] subcommand ...

Subcommands:
#{list_subcommand}

Global Options:
EOS
    opts.on("-d DIR", "rrse directory (default: #{DEFAULT_DIR})") do|dir|
      @rrse_dir = Pathname(dir)
    end
    opts
  end

  def run(argv)
    self.options.order!(argv)
    subcommand = argv.shift

    SUBCOMMANDS.fetch(subcommand, RRSE::Command::Unknown).new(@rrse_dir).run(argv)
  end

  
  def list_subcommand
    SUBCOMMANDS.map{|key, command_class|
      "  #{key.ljust(20)}#{command_class.short_description}"
    }.join("\n")
  end
end

class RRSE::Command::Unknown
  def initialize(dir); end
  def run(argv)
    RRSE::Main.show_help
  end
end
  
